#include "crypto.h"

#include "KISA_SHA256.h"

#define ARIA_BLOCKSIZE	16

crypto::crypto()
{
}

void crypto::setKey(cf::bin & key)
{
	CHEV_ARIA_Initialize(&info, CHEV_ARIA_256, key.buffer(), key.size());
}

cf::bin crypto::sha256(cf::bin & input)
{
	cf::bin out;

	out.resize(SHA256_DIGEST_VALUELEN);

	SHA256_Encrpyt(input.buffer(), input.size(), out.buffer());

	return out;
}

cf::bin crypto::encrypt(cf::bin & input)
{
	cf::bin plain;
	cf::bin out;
	int outSize;

	plain = input + getPaddingBytes(ARIA_BLOCKSIZE, input.size());

	out.resize(plain.size());

	CHEV_ARIA_Encrypt(&info, plain.buffer(), plain.size(), out.buffer(), &outSize);

	return out;
}

cf::bin crypto::decrypt(cf::bin & input)
{
	int plainSize;
	int paddingSize;
	cf::bin out;
	int outSize;

	out.resize(input.size());

	CHEV_ARIA_Decrypt(&info, input.buffer(), input.size(), out.buffer(), &outSize);

	paddingSize = out[outSize - 1];

	plainSize = outSize - paddingSize;
	out.resize(plainSize);

	return out;
}

cf::bin crypto::getPaddingBytes(int blockSize, int inputSize)
{
	int paddingSize = blockSize - (inputSize % blockSize);
	cf::bin result;

	result.resize(paddingSize);

	memset(result.buffer(), paddingSize, paddingSize);

	return result;
}
